# Copyright (C) 2011 Adam Stein <adam@csh.rit.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#------------------------------------------------------------------------
#
# Python modules
#
#------------------------------------------------------------------------
import datetime

#------------------------------------------------------------------------
#
# Gramps modules
#
#------------------------------------------------------------------------
from gen.lib import Date, DateError

#-------------------------------------------------------------------------
#
# Enhanced_Date class
#
#-------------------------------------------------------------------------
class Enhanced_Date(object):
    STYLE_ISO   = 0
    STYLE_US    = 1

    def __init__(self, date_obj):
        self.date_obj = date_obj
        self.style = Enhanced_Date.STYLE_ISO

    def get_style(self):
        """
        Return an integer indicating the presentation style

        The valid values are::

          STYLE_ISO       = international date format (ISO)
          STYLE_US        = style used in the United States
        """

        return self.style

    def is_empty(self):
        """
        Return True if the date contains no information (empty text).
        """

        return (self.date_obj == Date.MOD_TEXTONLY and not self.date_obj.text) or \
               (self.date_obj.get_start_date()==Date.EMPTY
                and self.date_obj.get_stop_date()==Date.EMPTY)

    def set_style(self, val):
        """
        Set the style for the date.
        """

        self.style = val

    def __str__(self):
        """
        Produce a string representation of the Date object. 
        
        If the date is not valid, the text representation is displayed. If
        the date is a range or a span, a string in the form of
        '<date_style> - <date_style>' is returned. Otherwise, a string in
        the form of '<date_style>' is returned where <date_style> is the
        specified style.
        """
        if self.date_obj.quality == Date.QUAL_ESTIMATED:
            qual = "est "
        elif self.date_obj.quality == Date.QUAL_CALCULATED:
            qual = "calc "
        else:
            qual = ""

        if self.date_obj.modifier == Date.MOD_BEFORE:
            pref = "bef "
        elif self.date_obj.modifier == Date.MOD_AFTER:
            pref = "aft "
        elif self.date_obj.modifier == Date.MOD_ABOUT:
            pref = "abt "
        else:
            pref = ""

        if self.date_obj.newyear == Date.NEWYEAR_JAN1:
            ny = ""
        elif self.date_obj.newyear == Date.NEWYEAR_MAR1:
            ny = "Mar1"
        elif self.date_obj.newyear == Date.NEWYEAR_MAR25:
            ny = "Mar25"
        elif self.date_obj.newyear == Date.NEWYEAR_SEP1:
            ny = "Sep1"
        else:
            ny = "Err"

        if self.date_obj.calendar != Date.CAL_GREGORIAN:
            if ny:
                cal = " (%s,%s)" % (Date.calendar_names[self.date_obj.calendar], ny)
            else:
                cal = " (%s)" % Date.calendar_names[self.date_obj.calendar]
        else:
            if ny:
                cal = " (%s)" % ny
            else:
                cal = ""

        if self.date_obj.modifier == Date.MOD_TEXTONLY:
            val = self.date_obj.text
        elif self.date_obj.get_slash():
            val = "%04d/%d-%02d-%02d" % (
                self.date_obj.dateval[Date._POS_YR] - 1,
                (self.date_obj.dateval[Date._POS_YR]) % 10,
                self.date_obj.dateval[Date._POS_MON],
                self.date_obj.dateval[Date._POS_DAY])
        elif self.date_obj.modifier == Date.MOD_RANGE or self.date_obj.modifier == Date.MOD_SPAN:
            val = "%04d-%02d-%02d - %04d-%02d-%02d" % (
                self.date_obj.dateval[Date._POS_YR], self.date_obj.dateval[Date._POS_MON],
                self.date_obj.dateval[Date._POS_DAY], self.date_obj.dateval[Date._POS_RYR],
                self.date_obj.dateval[Date._POS_RMON], self.date_obj.dateval[Date._POS_RDAY])
        else:
            if self.style == Enhanced_Date.STYLE_ISO:
                val = "%04d-%02d-%02d" % (
                    self.date_obj.dateval[Date._POS_YR], self.date_obj.dateval[Date._POS_MON],
                    self.date_obj.dateval[Date._POS_DAY])
            elif self.style == Enhanced_Date.STYLE_US:
                if self.date_obj.get_year_valid() and \
                   self.date_obj.get_month_valid() and \
                   self.date_obj.get_day_valid():
                    fmt = "%B %d, %Y"
                    dt = datetime.date(
                        self.date_obj.dateval[Date._POS_YR],
                        self.date_obj.dateval[Date._POS_MON],
                        self.date_obj.dateval[Date._POS_DAY]
                    )
                elif self.date_obj.get_year_valid() and \
                     self.date_obj.get_month_valid():
                    fmt = "%B %Y"
                    dt = datetime.date(
                        self.date_obj.dateval[Date._POS_YR],
                        self.date_obj.dateval[Date._POS_MON],
                        1       # Ignored in format
                    )
                elif self.date_obj.get_year_valid():
                    fmt = "%Y"
                    dt = datetime.date(
                        self.date_obj.dateval[Date._POS_YR],
                        1,      # Ignored in format
                        1       # Ignored in format
                    )
                elif self.date_obj.get_month_valid() and \
                     self.date_obj.get_day_valid():
                    fmt = "%B %d"
                    dt = datetime.date(
                        1900,   # Ignored in format
                        self.date_obj.dateval[Date._POS_MON],
                        self.date_obj.dateval[Date._POS_DAY]
                    )
                elif self.date_obj.get_month_valid():
                    fmt = "%B"
                    dt = datetime.date(
                        1900,   # Ignored in format
                        self.date_obj.dateval[Date._POS_MON],
                        1       # Ignored in format
                    )
                else:
                    fmt = ""
                    dt = datetime.date(
                        1900,   # Ignored in format
                        1,      # Ignored in format
                        1       # Ignored in format
                    )

                val = dt.strftime(fmt)
            else:
                raise DateError("invalid style")

        return "%s%s%s%s" % (qual, pref, val, cal)

